function [peaks,troughs,state]=getdate(ets,cts,y,k_ets,k_cts,l_ets,l_cts,a_ets,a_cts)
% *********************************************************************
% Author: Don Harding                                                  
% Procedure getdate() produces business cycle dates                    
% The algorithm is described in                                        
% However if one is interested in censoring the cycle to meet NBER     
% criteria then it is preferable to use procedure FINDATE  below       
% "The Definition, Dating and Duration of Cycles" paper.               
%  In brief the program comprises two user defined procedures:         
%        ETS(z,a)    that returns a 1 if an expansion terminating      
%                           sequence starts at observation t and       
%                            zero otherwise.                           
%        CTS(z,a)     that returns a 1 if a contraction terminating    
%                           sequence starts at observation t and zero  
%                            otherwise.                                
%        The arguments of ETS and CTS are explained below              
%                                                                      
%  Commented examples of these procedures are provided                 
%    for dating rules such as OKUN's two quarters of negative growth.  
%                                                                      
% In addition to the two procedures outlined above, five               
% variables are passed to  procedure GETDATE().  They are              
%       y :--  the series to be dated                                  
%       k_ETS :--  determines the number of observations before t that 
%               are passed to ETS()                                    
%       k_CTS :--  determines the number of observations before t that 
%               are passed to CTS()                                    
%       l_ETS  :-- determines the number of observations after t that  
%                passed to ETS()                                       
%       l_CTS  :-- determines the number of observations after t that  
%                are passed to CTS() in the z vector                   
%       a_ETS :-- a vector of parrameters that is passed to the        
%                         ETS procedure                                
%       a_CTS :-- this is a vector of parrameters that is passed to the
%                         CTS procedure                                


%                                                                      
%      The vector z that is passed to the CTS and ETS procedures is    
%       obtained as                                                    
%       z=y[t-k:t+l]                                                   
%  OUTPUT                                                              
%  The procedure has 3 t by 1 vector outputs                           
%       peaks:   Has 1 at peaks and zero elsewhere                     
%       troughs: Has 1 at troughs and zero elsewhere                   
%       state:   Has:-                                                 
%                   0 if state not known                               
%                   1 if state is expansion                            
%                  -1 if state is contraction                          
% *********************************************************************
% Number of observations in data series y 
% Counter for loops                       
% State vector 0=undetermined             
%              1=expansion               
%             -1=contraction             
% "Boolean"    1 if first terminating     
%                not found               
%              0 Otherwise               
% 2 dimension vector                      
% TS_MAT[1]= 1 if ETS encountered        
%          = 0 Otherwise                 
% TS_MAT[2]= 1 if CTS encountered        
%          = 0 Otherwise                 
% T by 1 vector of ones and zeros         
% peaks[i]= 1 if peak at i               
%         = 0 Otherwise                  
% T by 1 vector of ones and zeros         
% Troughs[i]= 1 if trough at i           
%           = 0 Otherwise                
%local ets:proc
% Procedure to check for expansion        
% terminating sequences                  
%local cts:proc
% Procedure to check for contraction      
% terminating sequences                  
t=rows(y);
% Initialise parameters        
peaks=zeros(t,1);
% and vectors                  
troughs=zeros(t,1);                              
state=zeros(t,1);                            
firstts=1;
                            
% ******************************************************
% FIRST LOOP                                            
% This loop finds the first terminating sequence in     
% the data.  It then marks a peak and sets the state    
% ******************************************************

i=1;
while firstts==1;
     lb_ets=max([(i-k_ets);1]);
     lb_cts=max([(i-k_cts);1]);
     ub_ets=min([(i+l_ets);t]);
     ub_cts=min([(i+l_cts);t]);
     i_ets=ub_ets-lb_ets+1-l_ets;
     i_cts=ub_cts-lb_cts+1-l_cts;
     j_ets=1+k_ets;
     j_cts=1+k_cts;
     %ts_mat=ets(y(lb_ets:ub_ets),i_ets,j_ets,a_ets)...
     %          [;cts(y(lb_cts:ub_cts),i_cts,j_cts,a_cts)];
     ts_mat1=ets(y(lb_ets:ub_ets),i_ets,j_ets,a_ets);
     ts_mat2=cts(y(lb_cts:ub_cts),i_cts,j_cts,a_cts);
     ts_mat=[ts_mat1;
                ts_mat2];

        if ts_mat(1)==1;
         %Peaks[i]=1;  
          state(i+1)=-1;
          firstts=0;
        elseif ts_mat(2)==1;
%Troughs[i]=1;  
          state(i+1)=1;
          firstts=0;
        else
% o Nothin
        end;
      i=i+1;
      if i>(t-max([l_ets;l_cts]));
          break;
      end;
end;


% **************
% END FIRST LOOP
% **************




% ******************************************************
% SECOND LOOP                                           
% This loop starts where the first loop finishes.       
% The state is now known so the first stopping time part
% of the algorithm can be implemented                   
%    --- When in an expansion the algorithm looks for th
%          first expansion terminating sequence ETS     
%          it then marks a peak and sets the state to   
%          contraction                                  
%    --- When in a contraction the algoritm looks for th
%           first contraction terminating sequence CTS  
%           It then marks a trough and sets the state   
%           to expansion                                
% ******************************************************


while i<=t-1;


     lb_ets=max([(i-k_ets);1]);
     lb_cts=max([(i-k_cts);1]);
     ub_ets=min([(i+l_ets);t]);
     ub_cts=min([(i+l_cts);t]);
     i_ets=ub_ets-lb_ets+1-l_ets;
     i_cts=ub_cts-lb_cts+1-l_cts;
     j_ets=1+k_ets;
     j_cts=1+k_cts;


     ts_mat1=ets(y(lb_ets:ub_ets),i_ets,j_ets,a_ets);
     ts_mat2=cts(y(lb_cts:ub_cts),i_cts,j_cts,a_cts);
     ts_mat=[ts_mat1;
                ts_mat2];


        if((state(i)==1)&(ts_mat(1)==1));
          peaks(i)=1;
          state(i+1)=-1;
        elseif((state(i)==-1)&(ts_mat(2)==1));
          troughs(i)=1;
          state(i+1)=1;
        else
          state(i+1)=state(i);
        end;
      i=i+1;
end;
while i<=t-1;
    state(i+1)=state(i);
     i=i+1;
end;
end
