function ranks = rankt(V,direction)
% rankt - Generate excel style ranks for the elements in V, accounting for ties
%
% usage: ranks = rankt(V)  % ranks will be for an increasing ordering
% usage: ranks = rankt(V,direction)
%
% arguments: (input)
%  V   - any row or column vector, may be integer, character
%      or floating point. In the event of floats, only EXACT
%      replicates are considered when ties are looked for.
%
%  direction - 'ascend' or 'descend' generates ascending or
%      descending ranks respectively.
%
% arguments: (output)
%  ranks - vector of corresponding ranks for each element in V
%      Tied elements are given an average rank for all tied
%      elements.
%
%
% Example: (ascending ranks, the default)
%  V = round(rand(1,12)*5)
% % V =
% %   4   1   1   3   2   2   3   4   2   3   2   4
%
% rankt(V)
% % ans =
% %  11 1.5 1.5   8 4.5 4.5   8  11 4.5   8 4.5  11
%
%
% Example: (rankt is vectorized, so it is quite fast.)
% V = round(rand(1,1e6)*1000000);
% tic, R = rankt(V); toc
% % Elapsed time is 0.230143 seconds.
%
%
% Example: (Descending versus ascending ranks)
%  V = [9 8 7 7 6 3 1 2 3 8 8];
%
% [V;rankt(V,'descend');rankt(V,'ascend')]
% % ans =
% %    9  8   7   7   6   3   1   2   3   8   8
% %    1  3 5.5 5.5   7 8.5  11  10 8.5   3   3
% %   11  9 6.5 6.5   5 3.5   1   2 3.5   9   9
%
%
% See also: sort, unique
%
% Author: John D'Errico
% e-mail: woodchips@rochester.rr.com
% Release date: 5/12/2010

if (nargin < 1) || (nargin > 2)
  error('RANKT:incorrectarguments', ...
    'Improper number of arguments. 1 or 2 arguments are required.')
end

if ~isvector(V)
  error('RANKT:incorrectarguments', ...
    'V must be a row or column vector.')
end
Vsize = size(V);
V = V(:);

% default and error check for direction
if (nargin < 2) || isempty(direction)
  direction = 'ascend';
elseif ~ischar(direction)
  error('RANKT:incorrectarguments', ...
    'direction must be either ''ascend'' or ''descend'' if supplied.')
else
  valid = {'ascend' 'descend'};
  
  ind = strmatch(lower(direction),lower(valid));
  if isempty(ind)
    % No hit found
    error('RANKT:incorrectarguments', ...
      'direction must be either ''ascend'' or ''descend'' if supplied, or a valid shortening thereof.')
  else
    direction = valid{ind};
  end
end

% unique will do the sort, plus allow us to know if there are reps
[VS,I,J] = unique(V); %#ok

% count the replicate elements to deal with ties
Vcount = accumarray(J,1);
cumulativeVcount = cumsum(Vcount);
ranks = cumulativeVcount(J) - (Vcount(J) - 1)/2;

% ensure that Vranks is the same orientation as was V
ranks = reshape(ranks,Vsize);

% Are these supposed to be increasing ranks or decreasing?
if direction(1) == 'd'
  % descending, so swap order
  ranks = (numel(V) + 1) - ranks;
end


