function [p_t_stats,t_p_stats,p_t_long,t_p_long]=summary_stats(peaks,troughs,x,d)
% ************************************************************************
% Author: Don Harding                                                     
% This procedure summarizes the business cycle features of the data       
% These summaries are in the form of information on:                      
%   duration                                                              
%   amplitude                                                             
%   cumulative movement over phases                                       
%   excess movement relative to the triangle approximation                
%   These terms are explained in the the following papers:                
%      "Knowing the cycle"    D. Harding and A Pagan                      
%      "Dissceting the cycle" D. Harding and A Pagan                      
%                                                                         
% The procedure is designed to deal with the cases where                  
%   * peaks or throughs are missing                                       
%         - this can arise in strongly trended data                       
%   * there are too few peaks or troughs to form summary statistics       
%   Because it can deal with these problems the procedure is useful in         
%     simulations                                                         
% The procedure takes four inputs:                                        
%   Peaks:   Which is a number of peaks by 1 vector with the typical      
%             element the observation number of the peak                  
%   Troughs: Which is a number of troughs by 1 vector with the typical    
%             element the observation number of the trough                
%   x:       Which is a T by 1 vector containing the original data        
%             for levels data such as GDP this should be the log of the   
%             levels.  If data is not in log of levels form then need to  
%             think carefully about what it means                         
%   d:       Which is a T by 1 vector                                     
%             d[i] indicates the duration of the current phase at obs i   
%             d[i] is negative in contractions and positive in expansions 
%                                                                         
% The procedure has four outputs:                                         
%   P_T_Stats  is dimension number of valid PT comparisons by six         
%       The columns of P_T_Stats are:                                     
%          [1] duration of contraction                                    
%          [2] amplitude of contraction                                   
%          [3] triangle approximation to cumulative output loss           
%          [4] cumulative output loss                                     
%          [5] excess cumulative output loss relative to triangle approx  
%          [6] average excess cumulative output loss                      


%   T_P_stats  is dimension number of valid PT comparisons by six         
%       The columns of T_P_Stats are:                                     
%          [1] duration of expansion                                      
%          [2] amplitude of expansion                                     
%          [3] triangle approximation to cumulative output gain           
%          [4] cumulative output gain                                     
%          [5] excess cumulative output gain relative to triangle approx  
%          [6] average excess cumulative output gain                      
%                                                                         
%          Let N_Comp be the number of observations that fall in completed
%              phases                                                     
%                                                                         
%   P_T_Long is dimension N_Comp by seven                                 
%       The columns of P_T_Long are peak to trough:                       
%          [1] long differences (P_T_Delta)                               
%          [2] cumulative movement (P_T_Cum)                              
%          [3] Excess relative to triangle approx (P_T_Excess)            
%          [4] average long difference (P_T_Delta_AV)                     
%          [5] average cumulative movement to date (P_T_Cum_AV)           
%          [6] average excess movement (P_T_Excess_AV)                    
%          [7] duration to date (P_T_Dur)                                 
%                                                                         
%   T_P_Long is dimension N_Comp by seven                                 
%       The columns of P_T_Long are trough to peak:                       
%          [1] long differences (T_P_Delta)                               
%          [2] cumulative movement (T_P_Cum)                              
%          [3] Excess relative to triangle approx (T_P_Excess)            
%          [4] average long difference (T_P_Delta_AV)                     
%          [5] average cumulative movement to date (T_P_Cum_AV)           
%          [6] average excess movement (T_P_Excess_AV)                    
%          [7] duration to date (T_P_Dur)                                 
%                                                                         
%   P_T_Long and T_P_Long provide the data for recession recovery analysis
% ***********************************************************************
p_t_excess=[];
t_p_excess=[];
p_t_delta=[];
t_p_delta=[];
p_t_cum=[];
t_p_cum=[];

p_t_excess_av=[];
t_p_excess_av=[];
p_t_delta_av=[];
t_p_delta_av=[];
p_t_cum_av=[];
t_p_cum_av=[];
p_t_dur=[];
t_p_dur=[];

% ************************************************************************
% The next block of code deals with the problems that arise where there ar
% too few peaks or troughs in the data to obtain valid statistics         
% ************************************************************************


% **********************************************************************
% This sub block of code deals with the case where there are at most one
% peak or trough in the data                                            
% all of the RETUNS are set to missing values since no valid statistics 
% can be computed                                                       
% **********************************************************************
if((rows(peaks)==0) | (rows(troughs)==0));
     p_t_stats=miss(zeros(6,1),0);
     t_p_stats=p_t_stats;
     t_p_long=miss(0,0);
     p_t_long=miss(0,0);
     return
end;
% **********************************************************************
% END sub block                                                         
% **********************************************************************


% **********************************************************************
% This sub block deals with the case where there are exactly one peak   
% and one trough in the data                                            
% If a peak is first one valid peak to trough comparison is possible    
%   but a missing value must be returned for trough to peak comparison  
% If a trough is first the reverse is true                              
% **********************************************************************
if max((rows(peaks)==1) & (rows(troughs))==1);
      if peaks(1)<troughs(1);
% First turning point is a peak
         num_p_t=1;
% Number of peak to trough comparisons 
         num_t_p=0;
% Number of trough to peak comparisons 
         p_t_stats=troughs(1:num_p_t)-peaks(1:num_p_t);
%Peak to trough duration
         t_p_stats=miss(zeros(6,1),0);
% trough to peak missing
% Next step calculates peak to trough amplitude
         p_t_stats=[p_t_stats (x(troughs(1:num_p_t),1)-x(peaks(1:num_p_t),1))];
% ext step calculates triangle approximation
         p_t_stats=[p_t_stats (0.5*p_t_stats(:,1).*p_t_stats(:,2))];
% *******************************************************************
% The next sub block calculates statistics obsevation by ovservation 
% for the one completed contraction phase                            
% *******************************************************************
        z=sum(x(peaks(1):troughs(1))-x(peaks(1)));
        delta=x(peaks(1)+1:troughs(1))-x(peaks(1));
        cum=cumsum(delta);
        d_i=d(peaks(1)+1:troughs(1));
        excess=cum-0.5*delta-((p_t_stats(1,2)/p_t_stats(1,1)).*abs(d_i));
        p_t_delta=delta;
        p_t_cum=cum;
        p_t_excess=excess;
        p_t_dur=d_i;
        p_t_delta_av=(delta./abs(d_i));
        p_t_cum_av=(cum./abs(d_i));
        p_t_excess_av=(excess./abs(d_i));
% *************
% End Sub Block
% *************


% *******************************************************************
% ext line removes bias in sum of rectangles approximation to integra
% *******************************************************************
         p_t_stats=[p_t_stats (z-0.5*p_t_stats(:,2))];


% *******************************************************************
% Next two lines calculate excess and average excess                 
% *******************************************************************
        p_t_stats=[p_t_stats (p_t_stats(:,4)-p_t_stats(:,3))];
        p_t_stats=[p_t_stats (p_t_stats(:,5)./p_t_stats(:,1))];


% ndicates whether first turning pont was a trough
     
% P_T_Stats=P_T_Stats~1;    REMOVE IF NO LONGER JUSTIFIED 
      else
%       First turning point is a trough
        num_p_t=0;
% Number of peak to trough comparisons 
        num_t_p=1;
% Number of trough to peak comparisons 
        p_t_stats=miss(zeros(6,1),0);
% Peak to trough statistics missing    
% Next step calculates trough to peak durati
        t_p_stats=peaks(1:num_t_p)-troughs(1:num_t_p);
% Next step calculates trough to peak amplitude
        t_p_stats=[t_p_stats (x(peaks(1:num_t_p),1)-x(troughs(1:num_t_p),1))];
% The next step calculates triangle approximation
        t_p_stats=[t_p_stats (0.5*t_p_stats(:,1).*t_p_stats(:,2))];
% *******************************************************************
% The next sub block calculates statistics obsevation by observation 
% for the one completed expansion phase                              
% *******************************************************************
        z=sum(x(troughs(1):peaks(1))-x(troughs(1)));
        delta=x(troughs(1)+1:peaks(1))-x(troughs(1));
        cum=cumsum(delta);
        d_i=d(troughs(1)+1:peaks(1));
        excess=cum-0.5*delta-((t_p_stats(1,2)/t_p_stats(1,1)).*abs(d_i));
        t_p_delta=delta;
        t_p_cum=cum;
        t_p_excess=excess;
        t_p_dur=d_i;
        t_p_delta_av=(delta./abs(d_i));
        t_p_cum_av=(cum./abs(d_i));
        t_p_excess_av=(excess./abs(d_i));
% *************
% End Sub Block
% *************

% *******************************************************************
% ext line removes bias in sum of rectangles approximation to integra
% *******************************************************************
      t_p_stats=[t_p_stats (z-0.5*t_p_stats(:,2))];


% *******************************************************************
% Next two lines calculate excess and average excess                 
% *******************************************************************
      t_p_stats=[t_p_stats (t_p_stats(:,4)-t_p_stats(:,3))];
      t_p_stats=[t_p_stats (t_p_stats(:,5)./t_p_stats(:,1))];
    end;
    t_p_long=miss(0,0);
    p_t_long=miss(0,0);
    return;
end;
% ************************************************************************
% nd of Block                                                             
% ************************************************************************
% **********************************************************************
% This sub block deals with the case where there suffients peaks        
% and troughs in the data to allow valid comparisons                    
% **********************************************************************

% *******************************************
if peaks(1)<=troughs(1);
% First turning point is a peak               
% *******************************************


              num_p_t=rows(troughs);
% Number of peak to trough comparisons 
              num_t_p=rows(peaks)-1;
% Number of trough to peak comparisons 


% *****************************************************************
% Next two lines calculate durations                               
% *****************************************************************
     p_t_stats=troughs(1:num_p_t)-peaks(1:num_p_t);
     t_p_stats=peaks(2:num_t_p+1)-troughs(1:num_t_p);


% *****************************************************************
% Next two lines calculate amplitudes                              
% *****************************************************************
      p_t_stats=[p_t_stats (x(troughs(1:num_p_t),1)-x(peaks(1:num_p_t),1))];
     t_p_stats=[t_p_stats (x(peaks(2:num_t_p+1),1)-x(troughs(1:num_t_p),1))];


% *****************************************************************
% Next two lines calculate triangle approximation                  
% *****************************************************************
     p_t_stats=[p_t_stats (0.5*p_t_stats(:,1).*p_t_stats(:,2))];
     t_p_stats=[t_p_stats (0.5*t_p_stats(:,1).*t_p_stats(:,2))];




% *******************************************************************
% The next DO LOOP calculates statistics obsevation by observation
% for all completed contraction phases
% *******************************************************************
    i=1;
    z=[];
    while i<=num_p_t;
        z=[z; sum(x(peaks(i):troughs(i))-x(peaks(i)))];
        delta=x(peaks(i)+1:troughs(i))-x(peaks(i));
        cum=cumsum(delta);
        d_i=d(peaks(i)+1:troughs(i));
        excess=cum-0.5*delta-((p_t_stats(i,2)/p_t_stats(i,1)).*abs(d_i));
        p_t_delta=[p_t_delta; delta];
        p_t_cum=[p_t_cum; cum];
        p_t_excess=[p_t_excess; excess];
        p_t_dur=[p_t_dur; d_i];
        p_t_delta_av=[p_t_delta_av; (delta./abs(d_i))];
        p_t_cum_av=[p_t_cum_av;(cum./abs(d_i))];
        p_t_excess_av=[p_t_excess_av; (excess./abs(d_i))];
        i=i+1;
    end;

% *******************************************************************
% The next line removes the bias in the sum of rectangles
% approximation to the triangle
% *******************************************************************
   p_t_stats=[p_t_stats (z-0.5*p_t_stats(:,2))];
   
% *******************************************************************
% The next DO LOOP calculates statistics obsevation by observation
% for all completed expansion phases
% *******************************************************************
    i=1;
    z=[];
    while i<=num_t_p;
        z=[z; sum(x(troughs(i):peaks(i+1))-x(troughs(i)))];
        delta=x(troughs(i)+1:peaks(i+1))-x(troughs(i));
        cum=cumsum(delta);
        d_i=d(troughs(i)+1:peaks(i+1));
        excess=cum-0.5*delta-((t_p_stats(i,2)/t_p_stats(i,1)).*abs(d_i));
        t_p_delta=[t_p_delta; delta];
        t_p_cum=[t_p_cum; cum];
        t_p_excess=[t_p_excess; excess];
        t_p_dur=[t_p_dur; d_i];
        t_p_delta_av=[t_p_delta_av; (delta./abs(d_i))];
        t_p_cum_av=[t_p_cum_av;(cum./abs(d_i))];
        t_p_excess_av=[t_p_excess_av; (excess./abs(d_i))];
        i=i+1;
    end;
 
% *******************************************************************
% The next line removes the bias in the sum of rectangles
% approximation to the triangle
% *******************************************************************
   t_p_stats=[t_p_stats (z-0.5*t_p_stats(:,2))];
  
%*********************************************************************/
% Next two lines calculate excess  
%*********************************************************************/
p_t_stats=[p_t_stats (p_t_stats(:,4)-p_t_stats(:,3))];
t_p_stats=[t_p_stats (t_p_stats(:,4)-t_p_stats(:,3))];  

%*********************************************************************/
% Next two lines calculate average excess relative to triangle
% approximation
%*********************************************************************/
p_t_stats=[p_t_stats (p_t_stats(:,5)-p_t_stats(:,1))];
t_p_stats=[t_p_stats (t_p_stats(:,5)-t_p_stats(:,1))];  

else
    
      num_p_t=rows(troughs)-1;
                % Number of peak to trough comparisons 
      num_t_p=rows(peaks);
        % Number of trough to peak comparisons 


% *****************************************************************
% Next two lines calculate durations                               
% *****************************************************************
     p_t_stats=troughs(2:num_p_t+1)-peaks(1:num_p_t);
     t_p_stats=peaks(1:num_t_p)-troughs(1:num_t_p);


% *****************************************************************
% Next two lines calculate amplitudes                              
% *****************************************************************
      p_t_stats=[p_t_stats (x(troughs(2:num_p_t+1),1)-x(peaks(1:num_p_t),1))];
     t_p_stats=[t_p_stats (x(peaks(1:num_t_p),1)-x(troughs(1:num_t_p),1))];


% *****************************************************************
% Next two lines calculate triangle approximation                  
% *****************************************************************
     p_t_stats=[p_t_stats (0.5*p_t_stats(:,1).*p_t_stats(:,2))];
     t_p_stats=[t_p_stats (0.5*t_p_stats(:,1).*t_p_stats(:,2))];




% *******************************************************************
% The next DO LOOP calculates statistics obsevation by observation
% for all completed contraction phases
% *******************************************************************
    i=1;
    z=[];
    while i<=num_p_t;
        z=[z; sum(x(peaks(i):troughs(i+1))-x(peaks(i)))];
        delta=x(peaks(i)+1:troughs(i+1))-x(peaks(i));
        cum=cumsum(delta);
        d_i=d(peaks(i)+1:troughs(i+1));
        excess=cum-0.5*delta-((p_t_stats(i,2)/p_t_stats(i,1)).*abs(d_i));
        p_t_delta=[p_t_delta; delta];
        p_t_cum=[p_t_cum; cum];
        p_t_excess=[p_t_excess; excess];
        p_t_dur=[p_t_dur; d_i];
        p_t_delta_av=[p_t_delta_av; (delta./abs(d_i))];
        p_t_cum_av=[p_t_cum_av;(cum./abs(d_i))];
        p_t_excess_av=[p_t_excess_av; (excess./abs(d_i))];
        i=i+1;
    end;

% *******************************************************************
% The next line removes the bias in the sum of rectangles
% approximation to the triangle
% *******************************************************************
   p_t_stats=[p_t_stats (z-0.5*p_t_stats(:,2))];
   
% *******************************************************************
% The next DO LOOP calculates statistics obsevation by observation
% for all completed expansion phases
% *******************************************************************
    i=1;
    z=[];
    while i<=num_t_p;
        z=[z; sum(x(troughs(i):peaks(i))-x(troughs(i)))];
        delta=x(troughs(i)+1:peaks(i))-x(troughs(i));
        cum=cumsum(delta);
        d_i=d(troughs(i)+1:peaks(i));
        excess=cum-0.5*delta-((t_p_stats(i,2)/t_p_stats(i,1)).*abs(d_i));
        t_p_delta=[t_p_delta; delta];
        t_p_cum=[t_p_cum; cum];
        t_p_excess=[t_p_excess; excess];
        t_p_dur=[t_p_dur; d_i];
        t_p_delta_av=[t_p_delta_av; (delta./abs(d_i))];
        t_p_cum_av=[t_p_cum_av;(cum./abs(d_i))];
        t_p_excess_av=[t_p_excess_av; (excess./abs(d_i))];
        i=i+1;
    end;
 
% *******************************************************************
% The next line removes the bias in the sum of rectangles
% approximation to the triangle
% *******************************************************************
   t_p_stats=[t_p_stats (z-0.5*t_p_stats(:,2))];
  
%*********************************************************************/
% Next two lines calculate excess  
%*********************************************************************/
p_t_stats=[p_t_stats (p_t_stats(:,4)-p_t_stats(:,3))];
t_p_stats=[t_p_stats (t_p_stats(:,4)-t_p_stats(:,3))];  

%*********************************************************************/
% Next two lines calculate average excess relative to triangle
% approximation
%*********************************************************************/
p_t_stats=[p_t_stats (p_t_stats(:,5)-p_t_stats(:,1))];
t_p_stats=[t_p_stats (t_p_stats(:,5)-t_p_stats(:,1))]; 
end;
p_t_long=[p_t_delta p_t_cum p_t_excess p_t_delta_av p_t_cum_av p_t_excess_av p_t_dur];
t_p_long=[t_p_delta t_p_cum t_p_excess t_p_delta_av t_p_cum_av t_p_excess_av t_p_dur];    

end