%Alain Schlaepfer (2010)
%computes average comulative movement (actual, triangular approximation
% and excess) of reference cycle for a region
%See Pagan and Harding (2002) for reference

%Input used:
%1. reference cycle of each country as 'countryname'ref.mat (=output from
%bbq2008.m)
%3. for each country, a mat-file named 'countryname', containing a matrix
%named 'countryname' with gdp data in first column and IP data in third
%column
%Output: File commov, matrix 'commov' structure of matrix:
%rows=countries;
%column 1-3=booms; columns 4-6=recessions;
%columns: 1%4=actual, 2&5=triangular, 3&6=excess
clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     ADJUST CONTROL VARIABLES

%list of countries
names={'france'; 'italy'; 'spain'; 'portugal'; 'greece';...
'cyprus';'croatia'; 'slovenia'; 'serbia'; 'macedonia';...
'malta'; 'turkey'; 'israel';'lebanon';...
'jordan';'egypt'; 'algeria'; 'morocco'; 'tunisia'};

%choose sample over which to compute: smp=0: whole sample, smp=1: up to
%specified date, smp=2: after specified date;
smp=0;
%specify date (2010q1-specdate), e.g. specdate=37=>2000q4; irrelevant if
%smp=0;
specdate=37;
%choose method of after sample selection. afterselect=0: only include
%phases starting after specified date, afterselect=1: all phases ending
%after specified date; irrelevant if smp=0 or smp=1;
afterselect=1;

%IPuse=0: use GDP only
%IPuse=1: use IP if no data on GDP available
%IPuse=2: use IP only
IPuse=1;
%choose name of output file
savefile  = 'cummovallcountriesIP';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cummovbr=[];
cummovrr=[];
    %loop over countries
    for i=1:size(names,1);
        cummovbc=[];
        cummovrc=[];
        %load refcycle of region    
        ireg=[char(names(i)) 'TP'];
        load(ireg);
        regcycle=refindic;
        %load country data
        icon=char(names(i));
        load(icon)
        %extract gdp
        eval(['data =' icon ';']);
        gdp=data(:,1);
        %if no data on gdp available, try IP
        if IPuse==1;
            if sum(isnan(gdp))==size(gdp,1);
                gdp=data(:,3);
            end;
        end;
        %or use IP always
        if IPuse==2;
            gdp=data(:,3);
        end;
        %use only regcycle where data available
        if size(regcycle,1)>size(gdp,1);
            rcyc=regcycle((end-size(gdp,1)+1):end,:);
        else
            gdp=gdp((end-size(regcycle,1)+1):end,:);
            rcyc=regcycle;
        end;
        time=(1:1:size(gdp,1))';
        peaks=selif(time, rcyc==1);
        troughs=selif(time, rcyc==-1);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %selection of sample to evaluate
        if smp==1;
            peaks=peaks(peaks<=(size(gdp,1)-specdate));
            troughs=troughs(troughs<=(size(gdp,1)-specdate));
        end;
        if smp==2;
            if afterselect==0;
                peaks=peaks(peaks>(size(gdp,1)-specdate));
                troughs=troughs(troughs>(size(gdp,1)-specdate));
            else
                p1=peaks(peaks<=(size(gdp,1)-specdate));
                t1=troughs(troughs<=(size(gdp,1)-specdate));
                p1=max([p1; 0]);
                t1=max([t1; 0]);
                late=max(p1,t1);
                peaks=peaks(peaks>=late);
                troughs=troughs(troughs>=late);
            end;
        end;
        
        %check if phase exist
        if size(peaks,1)==0 || size(troughs,1)==0;
            cummovbc=[NaN NaN NaN];
            cummovrc=[NaN NaN NaN];
        else
            %if peak first
            if peaks(1)<troughs(1);
                %loop over recessions
                for r=1:size(troughs);
                    ccummov=zeros(1,3);
                    ampl=(gdp(troughs(r))-gdp(peaks(r)))/gdp(peaks(r));
                    dur=troughs(r)-peaks(r);
                    %triangle approximation
                    ccummov(1,2)=ampl*dur/2;
                    %actual cumulative movement 
                    comact=0;
                    for d=1:dur;
                        comact=comact+(gdp(peaks(r)+d)-gdp(peaks(r)));
                    end;
                    ccummov(1,1)=comact/gdp(peaks(r));
                    %excess cumulative movement
                    ccummov(1,3)=(ampl*dur/2-comact/gdp(peaks(r))+ampl/2)/dur;
                    cummovrc=[cummovrc; ccummov];
                end;
                if size(peaks,1)==1
                    cummovbc=[NaN NaN NaN];
                else
                    %loop over booms
                    for b=2:size(peaks);
                        ampl=(gdp(peaks(b))-gdp(troughs(b-1)))/gdp(troughs(b-1));
                        dur=peaks(b)-troughs(b-1);
                        %triangle approximation
                        ccummov(1,2)=ampl*dur/2;
                        %actual cumulative movement 
                        comact=0;
                        for d=1:dur;
                            comact=comact+(gdp(troughs(b-1)+d)-gdp(troughs(b-1)));
                        end;
                        ccummov(1,1)=comact/gdp(troughs(b-1));
                        %excess cumulative movement
                        ccummov(1,3)=(ampl*dur/2-comact/gdp(troughs(b-1))+ampl/2)/dur;
                        cummovbc=[cummovbc; ccummov];
                    end;
                end;
            else
                %trough first
                %loop over booms
                for b=1:size(peaks);
                    ampl=(gdp(peaks(b))-gdp(troughs(b)))/gdp(troughs(b));
                    dur=peaks(b)-troughs(b);
                    %triangle approximation
                    ccummov(1,2)=ampl*dur/2;
                    %actual cumulative movement 
                    comact=0;
                    for d=1:dur;
                        comact=comact+(gdp(troughs(b)+d)-gdp(troughs(b)));
                    end;
                    ccummov(1,1)=comact/gdp(troughs(b));
                    %excess cumulative movement
                    ccummov(1,3)=(ampl*dur/2-comact/gdp(troughs(b))+ampl/2)/dur;
                    cummovbc=[cummovbc; ccummov];
                end;
                if size(troughs,1)==1
                    cummovrc=[NaN NaN NaN];
                else
                    %loop over recessions
                    for r=2:size(troughs);
                        ampl=(gdp(troughs(r))-gdp(peaks(r-1)))/gdp(peaks(r-1));
                        dur=troughs(r)-peaks(r-1);
                        %triangle approximation
                        ccummov(1,2)=ampl*dur/2;
                        %actual cumulative movement 
                        comact=0;
                        for d=1:dur;
                            comact=comact+(gdp(peaks(r-1)+d)-gdp(peaks(r-1)));
                        end;
                        ccummov(1,1)=comact/gdp(peaks(r-1));
                        %excess cumulative movement
                        ccummov(1,3)=(ampl*dur/2-comact/gdp(peaks(r-1))+ampl/2)/dur;
                        cummovrc=[cummovrc; ccummov];
                    end;
                end
            end;
        end;
        %average cummov for country
        cummovbc=nanmean(cummovbc,1);
        cummovrc=nanmean(cummovrc,1);
        cummovbr=[cummovbr; cummovbc];
        cummovrr=[cummovrr; cummovrc];
    end;
    %average cummov of all countries
    cummovbr=nanmean(cummovbr,1);
    cummovrr=nanmean(cummovrr,1);
    cummovall=[cummovbr cummovrr];

save(savefile, 'cummovall','names');        
            